/**********************************************************************************/
//CookieCullerTreeUtils 1.2.0
//Dan Yamaoka
//Original code taken from [Mozilla Firefox 0.8] treeUtils.js
//Modified original code is hopefully commented with "//DY"
//
//History:
// 1.0 Original Release
// 1.2.0 Support new tree code for Mozilla 1.8 trunk based builds

//DY - Added protect/unprotect buttons
function DeleteAllFromTree
    (tree, view, table, deletedTable, removeButton, removeAllButton, protButton, unprotButton) {

  // remove items from table and place in deleted table
  for (var i=0; i<table.length; i++) {
//DY - only if unprotected
    if (!table[i].isProtected) {
      deletedTable[deletedTable.length] = table[i];
      table[i] = null;
    }
  }
//DY - Clear selection to stop possible auto selection of null cookie queries
  if (oldTreeCode) {
	tree.treeBoxObject.selection.clearSelection();
  } else {
	tree.view.selection.clearSelection();
  }
//DY - fix up tree
  // collapse list by removing all the null entries
  for (var j=0; j<table.length; j++) {
    if (table[j] == null) {
      var k = j;
      while ((k < table.length) && (table[k] == null)) {
        k++;
      }
      table.splice(j, k-j);
      view.rowCount -= k - j;
      tree.treeBoxObject.rowCountChanged(j, j - k);
    }
  }
  // update selection and/or buttons
  if (table.length) {
    // update selection to top
    if (oldTreeCode) {
    	tree.treeBoxObject.view.selection.select(0);
    } else {
    	tree.view.selection.select(0);
    }
    tree.treeBoxObject.ensureRowIsVisible(0);
    //if it exists is must already be protected
        document.getElementById(unprotButton).disabled = false;
        document.getElementById(protButton).disabled = true;
  } else {
    // disable all buttons
    document.getElementById(removeButton).disabled = true;
    document.getElementById(removeAllButton).disabled = true;
    document.getElementById(unprotButton).disabled = true;
    document.getElementById(protButton).disabled = true;
  }
}

function DeleteSelectedItemFromTree
    (tree, view, table, deletedTable, removeButton, removeAllButton, protButton, unprotButton) {

  var selections = GetTreeSelections(tree);
//DY - Clear selection to stop possible auto selection of null cookie queries
  if (oldTreeCode) {
  	tree.treeBoxObject.selection.clearSelection();
  } else {
  	tree.view.selection.clearSelection();
  }

  // remove selected items from list (by setting them to null) and place in deleted list
  for (var s=selections.length-1; s>= 0; s--) {
    var i = selections[s];
    deletedTable[deletedTable.length] = table[i];
    table[i] = null;
  }
  // collapse list by removing all the null entries
  for (var j=0; j<table.length; j++) {
    if (table[j] == null) {
      var k = j;
      while ((k < table.length) && (table[k] == null)) {
        k++;
      }
      table.splice(j, k-j);
      view.rowCount -= k - j;
      tree.treeBoxObject.rowCountChanged(j, j - k);
    }
  }
//DY - update selection and/or buttons
  if (table.length) {

//DY - update selection to previous (first of) selected position or bottom
    var nextSelection = (selections[0] < table.length) ? selections[0] : table.length-1;
    if (oldTreeCode) {
    	tree.treeBoxObject.view.selection.select(nextSelection);
    } else {
    	tree.view.selection.select(nextSelection);
    }
    tree.treeBoxObject.ensureRowIsVisible(nextSelection);
    if (table[nextSelection].isProtected) {
        document.getElementById(unprotButton).disabled = false;
        document.getElementById(protButton).disabled = true;
    } else {
        document.getElementById(unprotButton).disabled = true;
        document.getElementById(protButton).disabled = false;
    }
  } else {
    // disable buttons
    document.getElementById(removeButton).disabled = true;
    document.getElementById(removeAllButton).disabled = true;
    document.getElementById(unprotButton).disabled = true;
    document.getElementById(protButton).disabled = true;
  }
}

function GetTreeSelections(tree) {
  var selections = [];
  var select;
  
  if (oldTreeCode) {
  	select = tree.treeBoxObject.selection;
  } else {
  	select = tree.view.selection;
  }
  if (select) {
    var count = select.getRangeCount();
    var min = new Object();
    var max = new Object();
    for (var i=0; i<count; i++) {
      select.getRangeAt(i, min, max);
      for (var k=min.value; k<=max.value; k++) {
        if (k != -1) {
          selections[selections.length] = k;
        }
      }
    }
  }
  return selections;
}

function SortTree(tree, view, table, column, lastSortColumn, lastSortAscending, updateSelection) {

  // remember which item was selected so we can restore it after the sort
  var selections = GetTreeSelections(tree);
  var selectedNumber = selections.length ? table[selections[0]].number : -1;

  // determine if sort is to be ascending or descending
  var ascending = (column == lastSortColumn) ? !lastSortAscending : true;

  // do the sort or re-sort
  var compareFunc = function compare(first, second) {
    if (column=="isProtected") {
       return second[column].toString().localeCompare(first[column].toString());
    } else {
       return first[column].toLowerCase().localeCompare(second[column].toLowerCase());
    }
  }
  table.sort(compareFunc);
  if (!ascending)
    table.reverse();

  // restore the selection
  var selectedRow = -1;
  if (selectedNumber>=0 && updateSelection) {
    for (var s=0; s<table.length; s++) {
      if (table[s].number == selectedNumber) {
        // update selection
        // note: we need to deselect before reselecting in order to trigger ...Selected()
        if (oldTreeCode) {
        	tree.treeBoxObject.view.selection.select(-1);
        	tree.treeBoxObject.view.selection.select(s);
        } else {
        	tree.view.selection.select(-1);
        	tree.view.selection.select(s);
        }
        selectedRow = s;
        break;
      }
    }
  }

  // display the results
  tree.treeBoxObject.invalidate();
  if (selectedRow >= 0) {
    tree.treeBoxObject.ensureRowIsVisible(selectedRow)
  }

  return ascending;
}

//DY - All new functions below
function ProtectInTree
    (tree, view, table, protButton, unprotButton, removeButton) {

  var selections = GetTreeSelections(tree);
  for (var s=selections.length-1; s>= 0; s--) {
    var i = selections[s];
    table[i].isProtected = true;
  }

  //update tree view
  tree.treeBoxObject.invalidate();
//DY - Update selections
  tree.treeBoxObject.ensureRowIsVisible(selections[0]);
  // disable/enable buttons
  document.getElementById(unprotButton).disabled = false;
  document.getElementById(protButton).disabled = true;
  document.getElementById(removeButton).disabled = true;
}

function UnProtectInTree
    (tree, view, table, protButton, unprotButton, removeButton) {

  var selections = GetTreeSelections(tree);
  for (var s=selections.length-1; s>= 0; s--) {
    var i = selections[s];
    table[i].isProtected = false;
  }

  //update tree view
  tree.treeBoxObject.invalidate();
//DY - Update selections
  tree.treeBoxObject.ensureRowIsVisible(selections[0]);
  // disable/enable buttons
  document.getElementById(unprotButton).disabled = true;
  document.getElementById(protButton).disabled = false;
  document.getElementById(removeButton).disabled = false;
}
